(* Name:
 * CS611
 *)

structure LambdaTranslate : LAMBDA_TRANSLATE = 
  struct
    
    local open LambdaAst 
    in
      
      exception LambdaTranslate of string
      
      fun translate (Var v) = Var v
	
	(* Primitive operations *)
	
	| translate (Num i) = (Num i)
	
	| translate (Op Plus) = 
	let 	
	  val x = newvar "x";
	  val y = newvar "y";
	in 
	  Lambda([x],Lambda([y],App [Op Plus, Var x, Var y]))
	end
  
	| translate (Op Times) = 
	let 	
	  val x = newvar "x";
	  val y = newvar "y";
	in 
	  Lambda([x],Lambda([y],App [Op Times, Var x, Var y]))
	end
  
	| translate (Op Minus) = 
	let 	
	  val x = newvar "x";
	  val y = newvar "y";
	in 
	  Lambda([x],Lambda([y],App [Op Minus, Var x, Var y]))
	end
  
	| translate (Op ZeroP) = (Op ZeroP)
    
	| translate (App ((Op Plus)::es)) =
	foldr (fn (e,e') => App [Op Plus, translate e, e'])
	(Num 0)
	(es)

	| translate (App ((Op Times)::es)) =
	foldr (fn (e,e') => App [Op Times, translate e, e'])
	(Num 1)
	(es)

	| translate (App ([Op Minus,e1,e2])) =
	App [Op Minus, translate e1, translate e2]

	(* Case operations *)

	| translate (Left e) = 
	let val x = newvar "x";
	  val y = newvar "y";
	in Lambda([x],Lambda([y], App[Var x, translate e]))
	end

	| translate (Right e) = 
	let val x = newvar "x";
	  val y = newvar "y";
	in Lambda([x],Lambda([y], App[Var y, translate e]))
	end

	| translate (Case(e,(v1,e1),(v2,e2))) =           
	let val e' = translate e;
	  val e1' = translate (Lambda ([v1],e1));
	  val e2' = translate (Lambda ([v2],e2));
	in App[App[e', e1'], e2']
	end


	(* Pair operations *)

	| translate (App[Op(Fst),e]) = 
	let val e' = translate e;
	  val x = newvar "x";
	  val y = newvar "y";
	in 
	  App[e', Lambda([x],Lambda([y],Var x))]
	end

	| translate (App[Op(Snd),e]) = 
	let val e' = translate e;
	  val x = newvar "x";
	  val y = newvar "y";
	in 
	  App[e', Lambda([x],Lambda([y],Var y))]
	end

	| translate (Pair (e1,e2)) = 
	let val f = newvar "f";
	in
	  translate (Lambda([f], App[Var f, e1, e2]))
	end

	(* Function operations *)

	| translate (Lambda ([], e)) = raise (LambdaTranslate "Lambda([]),e")

	| translate (Lambda ([x],e)) = 
	let 
	  val e' = translate e
	in 
	  Lambda ([x], e')
	end

	| translate (Lambda (v::vs,e)) = 
	let 
	  val e' = translate (Lambda (vs,e))
	in
	  Lambda([v],e')
	end

	| translate (App []) = raise (LambdaTranslate "App []")

	| translate (App [e]) = App[translate e]

	| translate (App (e::es)) = 
	foldl (fn (e,e') => App [e',translate e]) 
	(translate e)
	(es)

	(* Letrec *)

	| translate (Letrec (ves,e))=
	let 
	  (* lazy Y combinator *)
	  val Y = 
	    let val f = newvar "f";
	        val x = newvar "x";
	    in Lambda([f],
		      App[Lambda([x],App[Var f,App[Var x, Var x]]),
			  Lambda([x],App[Var f,App[Var x, Var x]])])
	    end
(*
	  (* strict Y combinator *)
	  val Y' = 
	    let val y = newvar "y";
	        val f = newvar "f";
	    in Lambda([y],Lambda([f],App[Var f,App[App[Var y,Var y],Var f]]))
	    end
	  val Y = App[Y',Y']
*)
(*
	  (* strict Y combinator *)
	  val Y =
	    let val f = newvar "f";
	        val x = newvar "x";
		val y = newvar "y";
	    in Lambda([f],App[Lambda([x],Lambda([y],App[Var f,
							App[Var x, Var x], 
							Var y])),
			      Lambda([x],Lambda([y],App[Var f,
							App[Var x, Var x], 
							Var y]))])
	    end
*)
	  val (vs,es) = ListPair.unzip ves;

	  (* List of arguments (g \v1...vn.e1) ... (g \v1...vn.en) *)
	  val g = newvar "g";
	  val y = newvar "y";
	  val args = map (fn exp => App[Var g, Lambda(vs,exp)]) es;

	  val F = Lambda([g],Lambda([y],App(Var y::args)));
	in
	  (* Apply F recursively to e abstracted over vs *)
	  translate (App[Y,F,Lambda(vs,e)])
	end

	| translate e = raise (LambdaTranslate "unimplemented")
	
      val transTrue = translate astTrue
      val transFalse = translate astFalse
    end
  end
