structure LPP = LambdaPPrint
structure LSPP = LambdaSimplePPrint
structure CPP = CPSPPrint

(* Use to parse a file in input language for HW3 into core languge for HW3 *)
fun load_lambda f 
  = let 
      val _ = Compiler.PPTable.install_pp 
	      ["LambdaAst", "expr"] LPP.ppExpr
      val _ = Compiler.PPTable.install_pp 
	      ["LambdaSimpleAst", "expr"] LSPP.ppExpr
      val _ = Compiler.PPTable.install_pp 
	      ["CPSAst", "exp"] CPP.ppExp
      val _ = Compiler.PPTable.install_pp 
	      ["CPSAst", "stm"] CPP.ppStm
      val e = LambdaParser.parse f
    in
      LambdaAst.alpha_vary e
    end

(* Use to run program in core language for HW3 *)
fun run_lambda e 
  = LambdaInterpretation.interpret
    (LambdaAst.alpha_vary e)

(* Use to parse a file in input langauge for HW3 into core language for HW3
   and run program in core language for HW3 *)
fun test_lambda f 
  = (LPP.ppE(run_lambda(load_lambda f)); 
     print "\n")


(* Use to translate program in core language for HW3 into target langauge for HW3 *)
fun lambda_translate e 
  = LambdaAst.alpha_vary 
    (LambdaTranslate.translate
     (LambdaAst.alpha_vary e))

(* Use to run a program in target language for HW3 *)
fun run_lambda_translate e 
  = LambdaInterpretation.interpret2
    (LambdaAst.alpha_vary e)

(* Use to parse a file in input langauge for HW3 into core language for HW3
   and translate program in core language for HW3 into target language for HW3
   and run program in target language for HW3 *)
fun test_lambda_translate f 
  = (LPP.ppE(run_lambda_translate(lambda_translate(load_lambda f))); 
     print "\n")


(* Use to parse a file in input language for HW5 into source languge for HW5 *)
fun load_lambda_simple f 
  = let
      val _ = Compiler.PPTable.install_pp 
	      ["LambdaAst", "expr"] LPP.ppExpr
      val _ = Compiler.PPTable.install_pp 
	      ["LambdaSimpleAst", "expr"] LSPP.ppExpr
      val _ = Compiler.PPTable.install_pp 
	      ["CPSAst", "exp"] CPP.ppExp
      val _ = Compiler.PPTable.install_pp 
	      ["CPSAst", "stm"] CPP.ppStm
      val e = LambdaSimpleParser.parse f
    in
      LambdaSimpleAst.alpha_vary e
    end

(* Use to run program in source language for HW5 *)
fun run_lambda_simple e 
  = LambdaSimpleInterpretation.interpret
    (LambdaSimpleAst.alpha_vary e)

(* Use to parse a file in input langauge for HW5 into source language for HW5
   and run program in source language for HW5 *)
fun test_lambda_simple f 
  = (LSPP.ppE(run_lambda_simple(load_lambda_simple f)); 
     print "\n")


(* Use to translate a program in target language for HW3 into source language for HW5 *)
fun convert e = LambdaSimpleAst.alpha_vary
                (LambdaSimpleAst.fromLambdaAst 
		 (LambdaAst.alpha_vary e))

(* Use to translate program in core language for HW3 into target language for HW3
   and translate program in target language for HW3 into source language for HW5 *)
fun lambda_translate_convert e 
  = convert(lambda_translate e)

(* Use to parse a file in input langauge for HW3 into core language for HW3
   and translate program in core langauge for HW3 into target langauge for HW3
   and translate program in target langauge for HW3 into source langauge for HW5
   and run program in source language for HW5 *)
fun test_lambda_translate_convert f 
  = (LSPP.ppE(run_lambda_simple(lambda_translate_convert(load_lambda f))); 
     print "\n")


(* Use to translate program in source language for HW5 into cps langauge for HW5 *)
fun cps_translate e
  = CPSTranslate.cps_translate
    (LambdaSimpleAst.alpha_vary e)

(* Use to run program in cps language for HW5 *)
fun run_cps s
  = (CPSEvaluation.evaluate s
     handle (exn as CPSEvaluation.CpsFailure s) => (print s;
						    print "\n";
						    raise exn))

(* Use to parse a file in input langauge for HW5 into source language for HW5
   and translate program in source language for HW5 into cps language for HW5
   and run program in cps language for HW5 *)
fun test_cps_translate f 
  = (CPP.ppE(run_cps(cps_translate(load_lambda_simple f))); 
     print "\n")


(* Use to parse a file in input langauge for HW3 into core language for HW3
   and translate program in core language for HW3 into target language for HW3
   and translate program in target language for HW3 into source language for HW5
   and translate program in source language for HW5 into cps language for HW5
   and run program in cps language for HW5 *)
fun test_lambda_translate_convert_cps_translate f
  = (CPP.ppE(run_cps(cps_translate(lambda_translate_convert(load_lambda f))));
     print "\n")
